<?php

define('VIEWS_ARGUMENTS_FROM_CALLBACK_COMMENT', 'Implements views:default_argument:from_callback');

class views_arguments_plugin_argument_default_from_callback extends views_plugin_argument_default {

  function option_definition() {
    $options = parent::option_definition();
    $options['callback'] = array('default' => '');

    return $options;
  }

  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['callback'] = array(
      '#type' => 'textfield',
      '#title' => t('Callback function'),
      '#default_value' => $this->options['callback'],
      '#description' => t("Enter an existing callback function. Make sure the callback function's comment includes <code>&quot;@comment&quot;</code>.", array('@comment' => VIEWS_ARGUMENTS_FROM_CALLBACK_COMMENT)),
    );
  }

  function options_validate(&$form, &$form_state) {
    $options = $form_state['values']['options'];
    if ($options['default_action'] == 'default' && $options['default_argument_type'] == 'from_callback') {
      $element = &$form['callback'];
      $callback = $element['#value'];
      if (!$this->validate_callback_function($callback, $reason)) {
        return form_error($element, $reason);
      }
    }
  }

  function get_argument() {
    $callback = $this->options['callback'];
    if ($this->validate_callback_function($callback)) {
      $argument = $callback($this->view, $this);
      return $argument;
    }
  }

  function validate_callback_function($callback, &$reason = NULL) {
    if (!$callback || !function_exists($callback)) {
      $reason = t('This function does not exist.');
      return FALSE;
    }

    $reflection = new ReflectionFunction($callback);
    $comment = $reflection->getDocComment();
    if (!$comment || !stristr($comment, VIEWS_ARGUMENTS_FROM_CALLBACK_COMMENT)) {
      $reason = t("This function's comment MUST include <code>&quot;@comment&quot;</code> to make sure it's yours.", array('@comment' => VIEWS_ARGUMENTS_FROM_CALLBACK_COMMENT));
      return FALSE;
    }

    return TRUE;
  }

}
